@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -310,7 +310,7 @@ function plugin_installed($plugin) {
 }
 
 function repair_database($run = true) {
-	global $altersopt;
+	global $altersopt, $database_default;
 
 	$alters = report_audit_results(false);
 
@@ -321,14 +321,20 @@ function repair_database($run = true) {
 		foreach($alters as $table => $changes) {
 			$tblinfo = db_fetch_row_prepared('SELECT ENGINE, SUBSTRING_INDEX(TABLE_COLLATION, "_", 1) AS COLLATION
 				FROM information_schema.tables
-				WHERE TABLE_SCHEMA="cacti"
+				WHERE TABLE_SCHEMA = ?
 				AND TABLE_NAME = ?',
-				array($table));
+				array($database_default, $table));
 
-			if ($tblinfo['ENGINE'] = 'MyISAM') {
-				$suffix = ",\n   ENGINE=InnoDB ROW_FORMAT=Dynamic CHARSET=" . $tblinfo['COLLATION'];
+			if (isset($tblinfo['COLLATION'])) {
+				$collation = $tblinfo['COLLATION'];
 			} else {
-				$suffix = ",\n   ROW_FORMAT=Dynamic CHARSET=" . $tblinfo['COLLATION'];
+				$collation = 'utf8mb4';
+			}
+
+			if ($tblinfo['ENGINE'] == 'MyISAM') {
+				$suffix = ",\n   ENGINE=InnoDB ROW_FORMAT=Dynamic CHARSET=" . $collation;
+			} else {
+				$suffix = ",\n   ROW_FORMAT=Dynamic CHARSET=" . $collation;
 			}
 
 			$sql = 'ALTER TABLE `' . $table . "`\n   " . implode(",\n   ", $changes) . $suffix . ';';
@@ -401,9 +407,9 @@ function report_audit_results($output = true) {
 			$status  = db_fetch_row('SHOW TABLE STATUS LIKE "' . $table_name . '"');
 
 			if ($status['Collation'] == 'utf8mb4_unicode_ci' || $status['Collation'] == 'utf8_general_ci') {
-				$text = 'mediumtext';
+				$collation = 'utf8';
 			} else {
-				$text = 'text';
+				$collation = 'latin';
 			}
 
 			if ($output) {
@@ -488,8 +494,8 @@ function report_audit_results($output = true) {
 						} else {
 							foreach($cols as $dbcol => $col) {
 								if ($col == 'Type' && $dbc[$dbcol] == 'text') {
-									if ($text == 'mediumtext') {
-										$dbc[$dbcol] = $text;
+									if ($collation == 'latin') {
+										$dbc[$dbcol] = 'mediumtext';
 									}
 								}
 
@@ -711,9 +717,14 @@ function report_audit_results($output = true) {
 function make_column_props(&$dbc) {
 	$alter_cmd = '';
 
-	$dbc['table_default'] = str_replace('current_timestamp()', 'CURRENT_TIMESTAMP', $dbc['table_default']);
-	$dbc['table_extra']   = str_replace('current_timestamp()', 'CURRENT_TIMESTAMP', $dbc['table_extra']);
-	$dbc['table_extra']   = trim(str_replace('DEFAULT_GENERATED', '', $dbc['table_extra']));
+	if (isset($dbc['table_default'])) {
+		$dbc['table_default'] = str_replace('current_timestamp()', 'CURRENT_TIMESTAMP', $dbc['table_default']);
+	}
+
+	if (isset($dbc['table_extra'])) {
+		$dbc['table_extra']   = str_replace('current_timestamp()', 'CURRENT_TIMESTAMP', $dbc['table_extra']);
+		$dbc['table_extra']   = trim(str_replace('DEFAULT_GENERATED', '', $dbc['table_extra']));
+	}
 
 	if ($dbc['table_null'] == 'YES') {
 		if ($dbc['table_default'] == 'NULL') {
