@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -65,7 +65,7 @@ switch (get_request_var('action')) {
 	break;
 }
 
-function manager(){
+function manager() {
 	global $config, $manager_actions, $item_rows;
 
 	/* ================= input validation and session storage ================= */
@@ -291,9 +291,9 @@ function manager_edit() {
 		print "<div class='tabs'><nav><ul role='tablist'>";
 
 		foreach (array_keys($tabs_manager_edit) as $tab_short_name) {
-			if (($id == 0 && $tab_short_name != 'general')){
+			if (($id == 0 && $tab_short_name != 'general')) {
 				print "<li class='subTab'><a href='#' " . (($tab_short_name == get_request_var('tab')) ? "class='selected'" : '') . "'>" . $tabs_manager_edit[$tab_short_name] . '</a></li>';
-			}else {
+			} else {
 				print "<li class='subTab'><a " . (($tab_short_name == get_request_var('tab')) ? "class='selected'" : '') .
 					" href='" . html_escape($config['url_path'] .
 					'managers.php?action=edit&id=' . get_request_var('id') .
@@ -322,7 +322,7 @@ function manager_edit() {
 		<?php }
 	}
 
-	switch(get_request_var('tab')){
+	switch(get_request_var('tab')) {
 		case 'notifications':
 			manager_notifications($id, $header_label);
 
@@ -362,11 +362,13 @@ function manager_edit() {
 
 	?>
 	<script language='javascript' type='text/javascript' >
+	$(function() {
 		$('.tooltip').tooltip({
 			track: true,
 			position: { collision: 'flipfit' },
-			content: function() { return $(this).attr('title'); }
+			content: function() { return DOMPurify.sanitize($(this).attr('title')); }
 		});
+	});
 	</script>
 	<?php
 }
@@ -439,7 +441,7 @@ function manager_notifications($id, $header_label) {
 	<script type='text/javascript'>
 
 	function applyFilter() {
-		strURL  = 'managers.php?action=edit&tab=notifications&id=<?php echo $id; ?>';
+		strURL  = 'managers.php?action=edit&tab=notifications&id=<?php print $id; ?>';
 		strURL += '&mib=' + $('#mib').val();
 		strURL += '&rows=' + $('#rows').val();
 		strURL += '&filter=' + $('#filter').val();
@@ -449,7 +451,7 @@ function manager_notifications($id, $header_label) {
 	}
 
 	function clearFilter() {
-		strURL = 'managers.php?action=edit&tab=notifications&id=<?php echo $id; ?>&clear=1&header=false';
+		strURL = 'managers.php?action=edit&tab=notifications&id=<?php print $id; ?>&clear=1&header=false';
 		loadPageNoHeader(strURL);
 	}
 
@@ -587,7 +589,7 @@ function manager_notifications($id, $header_label) {
 
 			if ($item['description']) {
 				print '<td><a href="#" title="<div class=\'header\'>' . $name . '</div><div class=\'content preformatted\'>' . $item['description']. '</div>" class="tooltip">' . $name . '</a></td>';
-			}else {
+			} else {
 				form_selectable_cell($name, $row_id);
 			}
 
@@ -707,10 +709,10 @@ function manager_logs($id, $header_label) {
 		div.style.display = 'none';
 	}
 
-	function highlightStatus(selectID){
+	function highlightStatus(selectID) {
 		if ($('#status_' + selectID).val() == 'ON') {
 			$('#status_' + selectID).css('background-color', 'LawnGreen');
-		}else {
+		} else {
 			$('#status_' + selectID).css('background-color', 'OrangeRed');
 		}
 	}
@@ -820,7 +822,7 @@ function manager_logs($id, $header_label) {
 					$description .= html_escape(trim($line)) . '<br>';
 				}
 				print '<td><a href="#" onMouseOut="hideTooltip(snmpagentTooltip)" onMouseMove="showTooltip(event, snmpagentTooltip, \'' . $item['notification'] . '\', \'' . $description . '\')">' . $item['notification'] . '</a></td>';
-			}else {
+			} else {
 				print "<td>{$item['notification']}</td>";
 			}
 			print "<td>$varbinds</td>";
@@ -854,7 +856,7 @@ function form_save() {
 	}
 	/* ================= input validation ================= */
 
-	switch(get_nfilter_request_var('tab')){
+	switch(get_nfilter_request_var('tab')) {
 		case 'notifications':
 			header('Location: managers.php?action=edit&tab=notifications&id=' . get_request_var('id'));
 			break;
@@ -871,9 +873,9 @@ function form_save() {
 			if ($save['snmp_version'] == 3) {
 				$save['snmp_username']        = form_input_validate(get_nfilter_request_var('snmp_username'), 'snmp_username', '', true, 3);
 				$save['snmp_password']        = form_input_validate(get_nfilter_request_var('snmp_password'), 'snmp_password', '', true, 3);
-				$save['snmp_auth_protocol']   = form_input_validate(get_nfilter_request_var('snmp_auth_protocol'), 'snmp_auth_protocol', "^\[None\]|MD5|SHA$", true, 3);
+				$save['snmp_auth_protocol']   = form_input_validate(get_nfilter_request_var('snmp_auth_protocol'), 'snmp_auth_protocol', "^\[None\]|MD5|SHA|SHA224|SHA256|SHA392|SHA512$", true, 3);
 				$save['snmp_priv_passphrase'] = form_input_validate(get_nfilter_request_var('snmp_priv_passphrase'), 'snmp_priv_passphrase', '', true, 3);
-				$save['snmp_priv_protocol']   = form_input_validate(get_nfilter_request_var('snmp_priv_protocol'), 'snmp_priv_protocol', "^\[None\]|DES|AES128$", true, 3);
+				$save['snmp_priv_protocol']   = form_input_validate(get_nfilter_request_var('snmp_priv_protocol'), 'snmp_priv_protocol', "^\[None\]|DES|AES|AES128|AES192|AES192C|AES256|AES256C$", true, 3);
 				$save['snmp_engine_id']       = form_input_validate(get_request_var_post('snmp_engine_id'), 'snmp_engine_id', '', false, 3);
 			} else {
 				$save['snmp_username']        = '';
@@ -908,14 +910,19 @@ function form_save() {
 	header('Location: managers.php?action=edit&header=false&id=' . (empty($manager_id) ? get_nfilter_request_var('id') : $manager_id) );
 }
 
-function form_actions(){
+function form_actions() {
 	global $manager_actions, $manager_notification_actions;
 
 	if (isset_request_var('selected_items')) {
 		if (isset_request_var('action_receivers')) {
-			$selected_items = sanitize_unserialize_selected_items(get_nfilter_request_var('selected_items'));
+			$selected_items = cacti_unserialize(stripslashes(get_nfilter_request_var('selected_graphs_array')));
+
+			if ($selected_items !== false) {
+				/* validate the selected items are ids */
+				foreach($selected_items as $index => $id) {
+					input_validate_input_number($id);
+				}
 
-			if ($selected_items != false) {
 				if (get_nfilter_request_var('drp_action') == '1') { // delete
 					db_execute('DELETE FROM snmpagent_managers WHERE id IN (' . implode(',' ,$selected_items) . ')');
 					db_execute('DELETE FROM snmpagent_managers_notifications WHERE manager_id IN (' . implode(',' ,$selected_items) . ')');
@@ -934,7 +941,7 @@ function form_actions(){
 			get_filter_request_var('id');
 			/* ==================================================== */
 
-			$selected_items = unserialize(stripslashes(get_nfilter_request_var('selected_items')), array('allowed_classes' => false));
+			$selected_items = cacti_unserialize(stripslashes(get_nfilter_request_var('selected_items')));
 
 			if ($selected_items !== false) {
 				if (get_nfilter_request_var('drp_action') == '1') { // disable
@@ -963,7 +970,7 @@ function form_actions(){
 			header('Location: managers.php?action=edit&id=' . get_nfilter_request_var('id') . '&tab=notifications&header=false');
 			exit;
 		}
-	}else {
+	} else {
 		if (isset_request_var('action_receivers')) {
 			$selected_items = array();
 			$list = '';
@@ -1023,7 +1030,7 @@ function form_actions(){
 			form_end();
 
 			bottom_footer();
-		}else {
+		} else {
 			$selected_items = array();
 			$list = '';
 
