@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -110,8 +110,7 @@ if (!cacti_sizeof($templates)) {
 if (timeToRun()) {
 	debug('Starting Spikekill Process');
 
-	list($micro,$seconds) = explode(' ', microtime());
-	$start   = $seconds + $micro;
+	$start   = microtime(true);
 
 	$graphs = kill_spikes($templates, $kills);
 
@@ -120,8 +119,7 @@ if (timeToRun()) {
 		$purges = purge_spike_backups();
 	}
 
-	list($micro,$seconds) = explode(' ', microtime());
-	$end  = $seconds + $micro;
+	$end  = microtime(true);
 
     $cacti_stats = sprintf(
         'Time:%01.4f ' .
@@ -152,22 +150,20 @@ function timeToRun() {
 	$lastrun   = read_config_option('spikekill_lastrun');
 	$frequency = read_config_option('spikekill_batch') * 3600;
 	$basetime  = strtotime(read_config_option('spikekill_basetime'));
-	$baseupper = 300;
-	$baselower = $frequency - 300;
+	$baseupper = $basetime + 300;
+	$baselower = $basetime - 300;
 	$now       = time();
 
 	debug("LastRun:'$lastrun', Frequency:'$frequency', BaseTime:'" . date('Y-m-d H:i:s', $basetime) . "', BaseUpper:'$baseupper', BaseLower:'$baselower', Now:'" . date('Y-m-d H:i:s', $now) . "'");
+
 	if ($frequency > 0 && ($now - $lastrun > $frequency)) {
 		debug("Frequency is '$frequency' Seconds");
 
-		$nowfreq = $now % $frequency;
-		debug("Now Frequency is '$nowfreq'");
-
-		if ((empty($lastrun)) && ($nowfreq > $baseupper) && ($nowfreq < $baselower)) {
+		if (empty($lastrun) && ($now < $baseupper) && ($now > $baselower)) {
 			debug('Time to Run');
 			db_execute_prepared('REPLACE INTO settings (name,value) VALUES ("spikekill_lastrun", ?)', array(time()));
 			return true;
-		} elseif (($now - $lastrun > 3600) && ($nowfreq > $baseupper) && ($nowfreq < $baselower)) {
+		} elseif (($now - $lastrun > $frequency) && ($now < $baseupper) && ($now > $baselower)) {
 			debug('Time to Run');
 			db_execute_prepared('REPLACE INTO settings (name,value) VALUES ("spikekill_lastrun", ?)', array(time()));
 			return true;
